@@ -62,6 +62,7 @@ typedef struct _php_smb_stream_data {
 	smbc_readdir_fn smbc_readdir;
 	smbc_write_fn smbc_write;
 	smbc_lseek_fn smbc_lseek;
+	smbc_ftruncate_fn smbc_ftruncate;
 } php_smb_stream_data;
 
 
@@ -127,7 +128,7 @@ static php_smbclient_state *php_smb_pool_get(php_stream_context *context, const
 		}
 	}
 
-	/* Crate a new state and save it in the pool */
+	/* Create a new state and save it in the pool */
 	pool = emalloc(sizeof(*pool));
 	memcpy(pool->hash, hash, 20);
 	pool->nb = 1;
@@ -145,8 +146,12 @@ static void php_smb_pool_drop(php_smbclient_state *state TSRMLS_DC)
 	for (pool = SMBCLIENT_G(pool_first); pool; pool = pool->next) {
 		if (pool->state == state) {
 			pool->nb--;
+			return;
 		}
 	}
+
+	/* Not found (after php_smb_pool_cleanup) so close it */
+	php_smbclient_state_free(state TSRMLS_CC);
 }
 
 void php_smb_pool_cleanup(TSRMLS_D) {
@@ -154,7 +159,9 @@ void php_smb_pool_cleanup(TSRMLS_D) {
 
 	pool = SMBCLIENT_G(pool_first);
 	while (pool) {
-		php_smbclient_state_free(pool->state TSRMLS_CC);
+		if (!pool->nb) { /* Keep it when still used */
+			php_smbclient_state_free(pool->state TSRMLS_CC);
+		}
 		pool = pool->next;
 		efree(pool);
 	}
@@ -169,6 +176,7 @@ static int php_smb_ops_close(php_stream *stream, int close_handle TSRMLS_DC)
 	if (!self) {
 		return EOF;
 	}
+
 	if (close_handle) {
 		if (self->handle) {
 			smbc_close = smbc_getFunctionClose(self->state->ctx);
@@ -190,7 +198,11 @@ static int php_smb_ops_flush(php_stream *stream TSRMLS_DC)
 	return 0;
 }
 
+#if PHP_VERSION_ID < 70400
 static size_t php_smb_ops_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+#else
+static ssize_t php_smb_ops_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+#endif
 {
 	ssize_t n = 0;
 	STREAM_DATA_FROM_STREAM();
@@ -208,12 +220,20 @@ static size_t php_smb_ops_read(php_stream *stream, char *buf, size_t count TSRML
 	if (n == 0 || n < (ssize_t)count) {
 		stream->eof = 1;
 	}
+#if PHP_VERSION_ID < 70400
 	return (n < 1 ? 0 : (size_t)n);
+#else
+	return n;
+#endif
 }
 
+#if PHP_VERSION_ID < 70400
 static size_t php_smb_ops_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
+#else
+static ssize_t php_smb_ops_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
+#endif
 {
-	size_t len = 0;
+	ssize_t len = 0;
 	STREAM_DATA_FROM_STREAM();
 
 	if (!self || !self->handle) {
@@ -225,7 +245,12 @@ static size_t php_smb_ops_write(php_stream *stream, const char *buf, size_t coun
 	if (self->smbc_write) {
 		len = self->smbc_write(self->state->ctx, self->handle, buf, count);
 	}
+
+#if PHP_VERSION_ID < 70400
+	return (len < 0 ? 0 : (size_t)len);
+#else
 	return len;
+#endif
 }
 
 static int php_smb_ops_stat(php_stream *stream, php_stream_statbuf *ssb TSRMLS_DC) /* {{{ */
@@ -262,6 +287,38 @@ static int php_smb_ops_seek(php_stream *stream, off_t offset, int whence, off_t
 	return -1;
 }
 
+static int php_smb_ops_set_option(php_stream *stream, int option, int value, void *ptrparam TSRMLS_DC)
+{
+	size_t newsize;
+	STREAM_DATA_FROM_STREAM();
+
+	if (!self || !self->handle) {
+		return PHP_STREAM_OPTION_RETURN_ERR;
+	}
+	if (!self->smbc_ftruncate) {
+		self->smbc_ftruncate = smbc_getFunctionFtruncate(self->state->ctx);
+	}
+	if (!self->smbc_ftruncate) {
+		return PHP_STREAM_OPTION_RETURN_ERR;
+	}
+
+	switch(option) {
+		case PHP_STREAM_OPTION_TRUNCATE_API:
+			switch (value) {
+				case PHP_STREAM_TRUNCATE_SUPPORTED:
+					return PHP_STREAM_OPTION_RETURN_OK;
+
+				case PHP_STREAM_TRUNCATE_SET_SIZE:
+					newsize = *(size_t*)ptrparam;
+					if (self->smbc_ftruncate(self->state->ctx, self->handle, newsize) == 0) {
+						return PHP_STREAM_OPTION_RETURN_OK;
+					}
+					return PHP_STREAM_OPTION_RETURN_ERR;
+			}
+	}
+	return PHP_STREAM_OPTION_RETURN_NOTIMPL;
+}
+
 static php_stream_ops php_stream_smbio_ops = {
 	php_smb_ops_write,
 	php_smb_ops_read,
@@ -271,7 +328,7 @@ static php_stream_ops php_stream_smbio_ops = {
 	php_smb_ops_seek,
 	NULL, /* cast */
 	php_smb_ops_stat,
-	NULL  /* set_option */
+	php_smb_ops_set_option
 };
 
 static php_stream *
@@ -500,7 +557,11 @@ static int php_smbdir_ops_close(php_stream *stream, int close_handle TSRMLS_DC)
 	return EOF;
 }
 
+#if PHP_VERSION_ID < 70400
 static size_t php_smbdir_ops_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+#else
+static ssize_t php_smbdir_ops_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+#endif
 {
 	struct smbc_dirent *dirent;
 	php_stream_dirent *ent = (php_stream_dirent*)buf;
