@@ -173,6 +173,14 @@ ZEND_BEGIN_ARG_INFO(arginfo_smbclient_option_set, 0)
 	ZEND_ARG_INFO(0, value)
 ZEND_END_ARG_INFO()
 
+#if HAVE_SMBC_SETOPTIONPROTOCOLS
+ZEND_BEGIN_ARG_INFO_EX(arginfo_smbclient_client_protocols, 0, 0, 1)
+	ZEND_ARG_INFO(0, state)
+	ZEND_ARG_INFO(0, minproto)
+	ZEND_ARG_INFO(0, maxproto)
+ZEND_END_ARG_INFO()
+#endif
+
 ZEND_BEGIN_ARG_INFO(arginfo_smbclient_path, 0)
 	ZEND_ARG_INFO(0, state)
 	ZEND_ARG_INFO(0, path)
@@ -267,6 +275,9 @@ static zend_function_entry smbclient_functions[] =
 	PHP_FE(smbclient_state_free, arginfo_smbclient_state)
 	PHP_FE(smbclient_option_get, arginfo_smbclient_option_get)
 	PHP_FE(smbclient_option_set, arginfo_smbclient_option_set)
+#if HAVE_SMBC_SETOPTIONPROTOCOLS
+	PHP_FE(smbclient_client_protocols, arginfo_smbclient_client_protocols)
+#endif
 	PHP_FE(smbclient_opendir, arginfo_smbclient_path)
 	PHP_FE(smbclient_readdir, arginfo_smbclient_dir)
 	PHP_FE(smbclient_closedir, arginfo_smbclient_dir)
@@ -608,6 +619,14 @@ php_smbclient_state_new (php_stream_context *context, int init TSRMLS_DC)
 				return NULL;
 			}
 		}
+#if HAVE_SMBC_SETOPTIONPROTOCOLS
+		if (NULL != (tmpzval = php_stream_context_get_option(context, "smb", "minproto"))) {
+			smbc_setOptionProtocols(state->ctx, Z_STRVAL_P(tmpzval), NULL);
+		}
+		if (NULL != (tmpzval = php_stream_context_get_option(context, "smb", "maxproto"))) {
+			smbc_setOptionProtocols(state->ctx, NULL, Z_STRVAL_P(tmpzval));
+		}
+#endif
 #else
 		zval **tmpzval;
 
@@ -629,6 +648,14 @@ php_smbclient_state_new (php_stream_context *context, int init TSRMLS_DC)
 				return NULL;
 			}
 		}
+#if HAVE_SMBC_SETOPTIONPROTOCOLS
+		if (php_stream_context_get_option(context, "smb", "minproto", &tmpzval) == SUCCESS) {
+			smbc_setOptionProtocols(state->ctx, Z_STRVAL_PP(tmpzval), NULL);
+		}
+		if (php_stream_context_get_option(context, "smb", "maxproto", &tmpzval) == SUCCESS) {
+			smbc_setOptionProtocols(state->ctx, NULL, Z_STRVAL_PP(tmpzval));
+		}
+#endif
 #endif
 	}
 	if (init) {
@@ -1345,15 +1372,17 @@ PHP_FUNCTION(smbclient_read)
 
 	if ((ZSTR_LEN(buf) = smbc_read(state->ctx, file, ZSTR_VAL(buf), count)) >= 0) {
 		RETURN_STR(buf);
+	}
+	zend_string_release(buf);
 #else
 	void *buf = emalloc(count);
 	ssize_t nbytes;
 
 	if ((nbytes = smbc_read(state->ctx, file, buf, count)) >= 0) {
 		RETURN_STRINGL(buf, nbytes, 0);
-#endif
 	}
 	efree(buf);
+#endif
 	switch (state->err = errno) {
 		case EISDIR: php_error(E_WARNING, "Read error: Is a directory"); break;
 		case EBADF: php_error(E_WARNING, "Read error: Not a valid file resource or not open for reading"); break;
@@ -1738,7 +1767,7 @@ PHP_FUNCTION(smbclient_removexattr)
 PHP_FUNCTION(smbclient_option_get)
 {
 	zend_long option;
-	char *ret;
+	const char *ret;
 	zval *zstate;
 	php_smbclient_state *state;
 
@@ -1946,6 +1975,23 @@ PHP_FUNCTION(smbclient_option_set)
 	RETURN_FALSE;
 }
 
+#if HAVE_SMBC_SETOPTIONPROTOCOLS
+PHP_FUNCTION(smbclient_client_protocols)
+{
+	zval *zstate;
+	char *minproto = NULL, *maxproto = NULL;
+	strsize_t minproto_len, maxproto_len;
+	php_smbclient_state *state;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|s!s!", &zstate, &minproto, &minproto_len, &maxproto, &maxproto_len) == FAILURE) {
+		return;
+	}
+	STATE_FROM_ZSTATE;
+
+	RETURN_BOOL(smbc_setOptionProtocols(state->ctx, minproto, maxproto));
+}
+#endif
+
 PHP_FUNCTION(smbclient_statvfs)
 {
 	char *url;
